@@ -1,11 +1,11 @@
 from datetime import datetime, timedelta
 from typing import List, Optional
-from feathr.definition.sink import Sink
+from feathr.definition.sink import HdfsSink, RedisSink, Sink
 import math
 
 
 class BackfillTime:
-    """Time range to materialize/backfill feature data. Please refer to https://linkedin.github.io/feathr/concepts/materializing-features.html#feature-backfill for a more detailed explanation.
+    """Time range to materialize/backfill feature data. Please refer to https://feathr-ai.github.io/feathr/concepts/materializing-features.html#feature-backfill for a more detailed explanation.
 
     Attributes:
         start: start time of the backfill, inclusive.
@@ -31,6 +31,9 @@ class MaterializationSettings:
         self.name = name
         now = datetime.now()
         self.backfill_time = backfill_time if backfill_time else BackfillTime(start=now, end=now, step=timedelta(days=1))
+        for sink in sinks:
+            if isinstance(sink, RedisSink):
+                sink.aggregation_features = feature_names
         self.sinks = sinks
         self.feature_names = feature_names
 
