@@ -5,7 +5,7 @@ from feathr.typed_key import TypedKey
 from feathr.source import HdfsSource
 
 batch_source = HdfsSource(name="nycTaxiBatchSource",
-                          path="abfss://feathrazuretest3fs@feathrazuretest3storage.dfs.core.windows.net/demo_data/green_tripdata_2020-04.csv",
+                          path="wasbs://public@azurefeathrstorage.blob.core.windows.net/sample_data/green_tripdata_2020-04.csv",
                           event_timestamp_column="lpep_dropoff_datetime",
                           timestamp_format="yyyy-MM-dd HH:mm:ss")
 
